% This program checks the optimality of the Ramsey rule relative to
% "corner" conditions in which we set the capital requirements to 0.
% In a static model we would only check one corner, but in a dynamic model,
% we need to check all possible combinations across time fo setting capital
% requirements to 0.
% To make the problem tractable, we check up to N periods (No_of_Bits, below), selected
% below. We then compute all possible combinations of periods in which the
% capital requirements could be set to 0 within this N periods. For example
% if N = 3, the combinations are:
% 1 0 0 
% 0 1 0 
% 0 0 1
% 1 1 0
% 0 1 1
% 1 0 1 
% 0 0 0
% 1 1 1 
% where 0 indicates whether in each period we set capital
% requirements to 0.
%
% N should be a large number. We have been able to set N (No_of_Bits) to 30.
%
% This program can check the optimality of the Ramsey rule for different
% shocks, which can be selected below.


%% set inputs for solution below

clear all

setpath_windows

global M_ oo_
global changes_names changes_vec

No_of_Bits = 20; %Horizon for checking zero capital requirements
max_switches = 3; %Maximum alternations in switches allowed

nperiods = 60;
maxiter = 30;

load_estimates = 0;  % choose a case for loading the estimated parameters, based on list below

% Select shock process

% shock for figure 3
irfshock = char('eps_a');
shockssequence = -1;

% shock for figure 4
%irfshock = char('eps_isp');
%shockssequence = 1;
     
% shock for figure 5
%irfshock = char('eps_tau');
%shockssequence = 1;    


% choose parameterization
% case 1 --- choose calibration for figures 1 through 5
% case 2 --- choose calibration 1 for tables 2 through 10
% case 3 --- look for weight of deposit in the utility function that makes
%            0 capital requirements optimal


mycase = 3;

if mycase == 1
    changes_names = char('gammap_increment');
    gammap_increment_val = 0.001;
    changes_vec = [gammap_increment_val];
    
elseif mycase == 2  % estimates based on a dataset that excludes stock prices
    % see program call_moment_matching.m
    changes_names = char('sigma_eps_a','sigma_eps_isp',...
        'rhop_a','rho_isp','phip_i','kappap','gammap_increment');
    
    load my_estimates.mat distance_param;
    gammap_increment_val = 0.001;
    changes_vec = [distance_param; gammap_increment_val];
        
elseif mycase == 3
    % no parameters loaded --- choices in the parameter file are left
    % unchanged
    
    changes_names = char('gammap_increment','sigmap_zero');
    changes_vec = [0.001 0.055];
    
    
end


mod_safe = 'model_calibration';    

mod_risky = 'model_gamma0';
                                                    
% end of inputs                                                                                                     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


[welf_opt, welf_vec, ds_vec, ktot_vec, ltot_vec, c_vec, welfc_vec, zdatass_, oobase_, Mbase_, violvecguess ] = ...
    solve_gamma0_check(mod_safe,mod_risky,...
    shockssequence,irfshock,nperiods,No_of_Bits,max_switches);

[max_welf_vec, max_welf_vec_pos] = max(welf_vec);

if max(welf_vec)>welf_opt
    display(['Welfare criterion is NOT satisfied: Setting capital requirements to zero becomes optimal.' ...
        ' Set zero capital requirements in those periods that match the position of 1 in the following sequence: ' ... 
        , num2str(violvecguess(max_welf_vec_pos,:))])
else
    display(['Welfare criterion is satisfied: The proposed path of endogenous capital requirements is optimal'])
end




