% This program replicates tables 5, 6, and 7 in "Optimal Dynamic Capital Requirements
% and Implementable Capital Buffer Rules" by Matthew Canzoneri, Behzad
% Diba, Luca Guerrieri and Arsenii Mishin.
%
% This program considers the change in the distance function relative to
% the results in call_tables_2_3_and_4.m from turning on volatility shocks
% using the same parameterization as for Figure 5.

clear all

setpath_windows

global M_ oo_
global changes_names changes_vec


modnam = 'model_calibration';

% load observed data for moment-matching exercise
data_table = readtable('calibration_dataset.xlsx');

% bandpass-filter the data
gdp_detrended = 100*bpass(log(data_table.gdp),6,32);
investment_detrended = 100*bpass(log(data_table.investment),6,32);
investment_price_detrended = 100*bpass(log(data_table.investment_price)-log(data_table.consumption_price),6,32);

dataset = [gdp_detrended(2:end) investment_detrended(2:end) investment_price_detrended(2:end)];
var_labels =char('GDP','Investment','Investment Price');

% select strategy for creating the GMM weighting matrix ---  needs to match
% choice in call_tables_2_3_and_4.m 
%optwt =1; % weighting matrix is diagonal optimal SMM matrix
optwt =2; % optimal GMM matrix
%optwt = -1; % weighting matrix is the identity matrix

% create the GMM weighting matrix
[data_momm,wtmat] = wtm_rm_noskew_opt(dataset,optwt);


% solve the model
eval(['dynare ',modnam,' noclearall'])

model_safe_paramfile;

% load my_estimates_sensitivity;
%init_theta = distance_param;

load my_estimates

init_theta = zeros(length(distance_param)+2,1);
init_theta(1:2) = distance_param(1:2);
init_theta(3) = sigma_eps_tau;
init_theta(4:5) = distance_param(3:4);
init_theta(6) = rho_taup;
init_theta(7:8) = distance_param(5:6);

thetaunc = init_theta;

% distance function from call_tables_2_3_and_4
changes_names = char('sigma_eps_a','sigma_eps_isp',...
    'rhop_a','rho_isp','phip_i','kappap');


[dist1 model_momm] = distance_function(init_theta([1 2 4 5 7 8]),data_momm,wtmat);

dist1


changes_names = char('sigma_eps_a','sigma_eps_isp','sigma_eps_tau',...
    'rhop_a','rho_isp','rho_taup','phip_i','kappap');


[dist2 model_momm] = distance_function_sensitivity(init_theta,data_momm,wtmat);

dist2

dist_difference = dist1-dist2



% create all the tables
nvars = size(var_labels,1);
this_mom = 0;
moment_labels = char('')

for i = 1:nvars
    for j=i:nvars
        this_mom = this_mom+1;
        if i == j
            moment_labels = strvcat(moment_labels,['Var(',deblank(var_labels(i,:)),')']);
            display(['Var(',deblank(var_labels(i,:)),') in the model is ',num2str(model_momm(this_mom)),' and in the data is ',num2str(data_momm(this_mom))])
        else
            moment_labels = strvcat(moment_labels,['Corr(',deblank(var_labels(i,:)),',', deblank(var_labels(j,:)),')']);
            display(['The covariance of ',deblank(var_labels(i,:)),' with ',deblank(var_labels(j,:)),' in the model is ',num2str(model_momm(this_mom)),' and in the data is ',num2str(data_momm(this_mom))])
            
        end
    end
end

for i = 1:nvars
    this_mom = this_mom+1;
    moment_labels = strvcat(moment_labels,['Autocorr(',deblank(var_labels(i,:)),')']);
    display(['The auto-covariance of ',deblank(var_labels(i,:)),' is ',num2str(model_momm(this_mom)),' and in the data is ',num2str(data_momm(this_mom))])
    
end

%%

changes_vec = init_theta;


nperiods = 2000;
irf_shock = char('eps_a','eps_isp','eps_tau');

randn('seed',1);
nshocks = size(irf_shock,1);
shockssequence = randn(nperiods,nshocks);

[zdatalinear, zdatass, oobase_, Mbase_ ] = ...
    solve_no_constraint_no_parse(shockssequence,irf_shock,nperiods);

ytot_pos = strmatch('ytot',Mbase_.endo_names,'exact');
ltot2ytot_pos = strmatch('ltot2ytot',Mbase_.endo_names,'exact');
gamma_var_pos = strmatch('gamma_var',Mbase_.endo_names,'exact');
ginvest_pos = strmatch('ginvest',Mbase_.endo_names,'exact');
q_pos = strmatch('q',Mbase_.endo_names,'exact');
r_e_pos = strmatch('r_e',Mbase_.endo_names,'exact');

f0_ytot_data = zdatalinear(:,ytot_pos);
f0_ytot_ss = zdatass(ytot_pos);
f0_ltot2ytot_data = zdatalinear(:,ltot2ytot_pos);
f0_ginvest_data = zdatalinear(:,ginvest_pos);
f0_ginvest_ss = zdatass(ginvest_pos);
f0_gamma_var_data = zdatalinear(:,gamma_var_pos);
f0_q_data = zdatalinear(:,q_pos);
f0_r_e_data = zdatalinear(:,r_e_pos);


gdp = 100*bpass(f0_ytot_data/f0_ytot_ss,6,32);
ltot2ytot = 100*bpass(f0_ltot2ytot_data,6,32);
gamma_var = 100*bpass(f0_gamma_var_data,6,32);
ginvest = 100*bpass(f0_ginvest_data/f0_ginvest_ss,6,32);
q = 100*bpass(f0_q_data,6,32);
r_e =100*bpass(f0_r_e_data,6,32);

tot_gdp_var = var(gdp);
tot_ltot2ytot = var(ltot2ytot);
tot_gamma_var = var(gamma_var);
tot_ginvest_var = var(ginvest);
tot_q_var = var(q);
tot_r_e_var = var(r_e);

% repeat the variance calculation only turning on the TFP shock
nperiods = 2000;
irf_shock = char('eps_a');


[zdatalinear, zdatass, oobase_, Mbase_ ] = ...
    solve_no_constraint_no_parse(shockssequence(:,1),irf_shock,nperiods);

ytot_pos = strmatch('ytot',Mbase_.endo_names,'exact');
gamma_var_pos = strmatch('gamma_var',Mbase_.endo_names,'exact');


f0_ytot_data = zdatalinear(:,ytot_pos);
f0_ytot_ss = zdatass(ytot_pos);
f0_ltot2ytot_data = zdatalinear(:,ltot2ytot_pos);
f0_ginvest_data = zdatalinear(:,ginvest_pos);
f0_ginvest_ss = zdatass(ginvest_pos);
f0_gamma_var_data = zdatalinear(:,gamma_var_pos);
f0_q_data = zdatalinear(:,q_pos);
f0_r_e_data = zdatalinear(:,r_e_pos);


gdp = 100*bpass(f0_ytot_data/f0_ytot_ss,6,32);
ltot2ytot = 100*bpass(f0_ltot2ytot_data,6,32);
gamma_var = 100*bpass(f0_gamma_var_data,6,32);
ginvest = 100*bpass(f0_ginvest_data/f0_ginvest_ss,6,32);
q = 100*bpass(f0_q_data,6,32);
r_e =100*bpass(f0_r_e_data,6,32);

part_gdp_var_a = var(gdp);
part_ltot2ytot_var_a = var(ltot2ytot);
part_gamma_var_a = var(gamma_var);
part_ginvest_var_a = var(ginvest);
part_q_var_a = var(q);
part_r_e_var_a = var(r_e);




% get simulated data with only the ISP shock turned on
nperiods = 2000;
irf_shock = char('eps_isp');

[zdatalinear, zdatass, oobase_, Mbase_ ] = ...
    solve_no_constraint_no_parse(shockssequence(:,2),irf_shock,nperiods);

ytot_pos = strmatch('ytot',Mbase_.endo_names,'exact');
gamma_var_pos = strmatch('gamma_var',Mbase_.endo_names,'exact');


f0_ytot_data = zdatalinear(:,ytot_pos);
f0_ytot_ss = zdatass(ytot_pos);
f0_ltot2ytot_data = zdatalinear(:,ltot2ytot_pos);
f0_ginvest_data = zdatalinear(:,ginvest_pos);
f0_ginvest_ss = zdatass(ginvest_pos);
f0_gamma_var_data = zdatalinear(:,gamma_var_pos);
f0_q_data = zdatalinear(:,q_pos);
f0_r_e_data = zdatalinear(:,r_e_pos);


gdp = 100*bpass(f0_ytot_data/f0_ytot_ss,6,32);
ltot2ytot = 100*bpass(f0_ltot2ytot_data,6,32);
gamma_var = 100*bpass(f0_gamma_var_data,6,32);
ginvest = 100*bpass(f0_ginvest_data/f0_ginvest_ss,6,32);
q = 100*bpass(f0_q_data,6,32);
r_e =100*bpass(f0_r_e_data,6,32);


part_gdp_var_isp = var(gdp);
part_ltot2ytot_var_isp = var(ltot2ytot);
part_gamma_var_isp = var(gamma_var);
part_ginvest_var_isp = var(ginvest);
part_q_var_isp = var(q);
part_r_e_var_isp = var(r_e);


% get simulated data with only the tau shock turned on
nperiods = 2000;
irf_shock = char('eps_tau');

[zdatalinear, zdatass, oobase_, Mbase_ ] = ...
    solve_no_constraint_no_parse(shockssequence(:,2),irf_shock,nperiods);

ytot_pos = strmatch('ytot',Mbase_.endo_names,'exact');
gamma_var_pos = strmatch('gamma_var',Mbase_.endo_names,'exact');


f0_ytot_data = zdatalinear(:,ytot_pos);
f0_ytot_ss = zdatass(ytot_pos);
f0_ltot2ytot_data = zdatalinear(:,ltot2ytot_pos);
f0_ginvest_data = zdatalinear(:,ginvest_pos);
f0_ginvest_ss = zdatass(ginvest_pos);
f0_gamma_var_data = zdatalinear(:,gamma_var_pos);
f0_q_data = zdatalinear(:,q_pos);
f0_r_e_data = zdatalinear(:,r_e_pos);


gdp = 100*bpass(f0_ytot_data/f0_ytot_ss,6,32);
ltot2ytot = 100*bpass(f0_ltot2ytot_data,6,32);
gamma_var = 100*bpass(f0_gamma_var_data,6,32);
ginvest = 100*bpass(f0_ginvest_data/f0_ginvest_ss,6,32);
q = 100*bpass(f0_q_data,6,32);
r_e =100*bpass(f0_r_e_data,6,32);


part_gdp_var_tau = var(gdp);
part_ltot2ytot_var_tau = var(ltot2ytot);
part_gamma_var_tau = var(gamma_var);
part_ginvest_var_tau = var(ginvest);
part_q_var_tau = var(q);
part_r_e_var_tau = var(r_e);

%%

sum_gdp_var = part_gdp_var_a + part_gdp_var_isp + part_gdp_var_tau;
sum_ltot2ytot_var = part_ltot2ytot_var_a + part_ltot2ytot_var_isp + part_ltot2ytot_var_tau;
sum_gamma_var = part_gamma_var_a + part_gamma_var_isp + part_gamma_var_tau;
sum_ginvest_var = part_ginvest_var_a + part_ginvest_var_isp + part_ginvest_var_tau;
sum_q_var = part_q_var_a + part_q_var_isp + part_q_var_tau;
sum_r_e_var = part_r_e_var_a + part_r_e_var_isp + part_r_e_var_tau;


%%

gdp_var_share = [round(100*part_gdp_var_a/sum_gdp_var)
    round(100*part_gdp_var_isp/sum_gdp_var)
    100-round(100*part_gdp_var_a/sum_gdp_var)-round(100*part_gdp_var_isp/sum_gdp_var)
    ];

ltot2ytot_var_share = [round(100*part_ltot2ytot_var_a/sum_ltot2ytot_var)
    round(100*part_ltot2ytot_var_isp/sum_ltot2ytot_var)
    100-round(100*part_ltot2ytot_var_a/sum_ltot2ytot_var)-round(100*part_ltot2ytot_var_isp/sum_ltot2ytot_var)
    ];

gamma_var_share =[round(100*part_gamma_var_a/sum_gamma_var)
    round(100*part_gamma_var_isp/sum_gamma_var)
    100-round(100*part_gamma_var_a/sum_gamma_var)-round(100*part_gamma_var_isp/sum_gamma_var)
    ];

ginvest_var_share =[round(100*part_ginvest_var_a/sum_ginvest_var)
    round(100*part_ginvest_var_isp/sum_ginvest_var)
    100-round(100*part_ginvest_var_a/sum_ginvest_var)-round(100*part_ginvest_var_isp/sum_ginvest_var)
    ];

q_var_share =[round(100*part_q_var_a/sum_q_var)
    round(100*part_q_var_isp/sum_q_var)
    100-round(100*part_q_var_a/sum_q_var)-round(100*part_q_var_isp/sum_q_var)
    ];


r_e_var_share =[round(100*part_r_e_var_a/sum_r_e_var)
    round(100*part_r_e_var_isp/sum_r_e_var)
    100-round(100*part_r_e_var_a/sum_r_e_var)-round(100*part_r_e_var_isp/sum_r_e_var)
    ];

%%
sideways_switch = 0;
caption = '';

texfilename = 'table_moment_sensitivity';
columnlabels = char('Data','Model');
rowlabels = moment_labels;
table_mat = [data_momm, model_momm];
column_precision = [2, 2];
maketable(table_mat,texfilename,columnlabels,rowlabels,sideways_switch,caption,column_precision)


texfilename = 'table_calibration_sensitivity';
columnlabels = char('AR(1) param.','Innov. St. Dev.');
rowlabels = char('TFP','ISP','Volatility$^*$');
table_mat = [thetaunc([4,5,6]), thetaunc([1,2,3])];
column_precision = [2, 4];
maketable(table_mat,texfilename,columnlabels,rowlabels,sideways_switch,caption,column_precision)


texfilename = 'table_calibration_share_sensitivity';
columnlabels = char('var(GDP)','var(invest.)','var(invest. p.)','var(gamma)','var(credit/GDP)');
rowlabels = char('TFP','ISP','Volatility');
table_mat_share =[gdp_var_share, ginvest_var_share, q_var_share, gamma_var_share, ltot2ytot_var_share];
column_precision = [0 0 0 0 0];
maketable(table_mat_share,texfilename,columnlabels,rowlabels,sideways_switch,caption,column_precision)



